package com.javahonk.jms.solace;

import javax.annotation.PostConstruct;
import javax.jms.Connection;
import javax.jms.ConnectionFactory;
import javax.jms.Destination;
import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.MessageConsumer;
import javax.jms.MessageListener;
import javax.jms.MessageProducer;
import javax.jms.Session;
import javax.jms.TextMessage;
import javax.jms.Topic;
import javax.naming.InitialContext;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;

import com.solacesystems.jms.SolJmsUtility;
import com.solacesystems.jms.message.SolBytesMessage;
import com.solacesystems.jms.message.SolTextMessage;

public class SolaceMessagePublisherSubscriber implements MessageListener {
	
	private static final Logger logger = LogManager.getLogger(SolaceMessagePublisherSubscriber.class.getName());

	@Autowired
	private SolaceConnectionFactory solaceConnectionFactory;
	
	@Autowired
    private InitialContext initialContext_publisher;
	
	@Autowired
    private InitialContext initialContext_subscriber;
	
	@Value("${SOLACE_OUTPUT_TOPIC_NAME}")
	private String SOLACE_OUTPUT_TOPIC_NAME;
	
	@Value("${SOLACE_JMS_Direct_Connection_Factory}")
	private String SOLACE_JMS_Direct_Connection_Factory;
	
	private Session session;
	
	private MessageProducer producer;
	
	private MessageConsumer consumer;
	
	@PostConstruct
	public void init() {
		
		logger.info("Solace outputQueue: {}", SOLACE_OUTPUT_TOPIC_NAME);
		
		Connection connection;
		try {
			
			//Publisher
			connection = solaceConnectionFactory.createPublisherConnection();			
			session = solaceConnectionFactory.createSession(connection);
			
			Destination destination = (Destination) initialContext_publisher.lookup(SOLACE_OUTPUT_TOPIC_NAME);
			producer = session.createProducer(destination);
			
			//Subscriber
			ConnectionFactory cf = (ConnectionFactory)initialContext_subscriber.lookup(SOLACE_JMS_Direct_Connection_Factory);
	    	Connection subsConnection = cf.createConnection();
	    	Session session = subsConnection.createSession(false, Session.CLIENT_ACKNOWLEDGE);
	        Topic topic = (Topic)initialContext_subscriber.lookup(SOLACE_OUTPUT_TOPIC_NAME);
	        consumer = session.createConsumer(topic);
	        consumer.setMessageListener(this);
	        
	        connection.start();
	        subsConnection.start(); 
	        
			
		} catch (Exception e) {
			logger.error("Error creating connection to Solace: {}", e);
		} 
	}
	
	public void sendMessage(String message) {
		
		try {
			
			logger.info("Sending message : {}", message);
			
			TextMessage testMessage = session.createTextMessage(message);
			producer.send(testMessage);
			
			logger.info("Message sent successfully : {}", message);
			
		} catch (Exception e) {
			logger.error("Error sending message to output queue name:{} --> details exceptions: --> ", SOLACE_OUTPUT_TOPIC_NAME, e);
		}
	}

	@Override
	public void onMessage(Message testMessage) {
		try {
        	logger.info("Message received");
            
            if (testMessage == null) {	            	
            	logger.info("An error has occured. Solace Text message was null");	            	
            }
            
            if (testMessage instanceof SolBytesMessage) {            	
            	System.out.println("Message received:"+((SolBytesMessage)testMessage).getMessage());
            	logger.info("Message size: {} kb",SolJmsUtility.getMessageSize(testMessage));            	            
            }
            
            if (testMessage instanceof SolTextMessage) {            	
            	System.out.println("Message received:"+((SolTextMessage)testMessage).getText());
            	logger.info("Message size: {} kb",SolJmsUtility.getMessageSize(testMessage));            	
            }
        } catch (JMSException e) {
            System.out.println("Error processing incoming message.");
            e.printStackTrace();
        }
		
	}

}
